/**
 * Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var Tas;
var data = require('../../core/data');
var units = require('../../core/units');

var nested = require('./nested');
var forEach = require('./forEach');
var logArray = require('./logArray');
var indent = require('./indent');
var util = require('./util');

var tree = {
	init: function (Tas_){
		Tas = Tas_;
		forEach.saveTree(this);

		this.logArray = logArray;
		this.nested = nested;
		this.forEach = forEach;

		Tas.tree = this;
		Tas.enableTree = this.enable;
		Tas.disableTree = this.disable;
		Tas.isTreeEnabled = this.isTreeEnabled;
	},

	enable: function(){
		data.extensions.isTreeEnabled = true;
	},

	disable: function(){
		data.extensions.isTreeEnabled = false;
	},

	isTreeEnabled: /* istanbul ignore next*/ function(){
		return data.extensions.isTreeEnabled;
	},

	print: function(func, args){
		var funcName = func.__isIgnore ? func.__label : (func.treeBranch || func.name || '<anonymous>');
		if (!funcName) return;

		if (args) {
			var str = '';
			var maxLen = 200; // 2 lines in terminal of mac os

			args.forEach(function(arg){
				typeof arg !== 'undefined' && (str += JSON.stringify(arg) + ', ');
			});

			str = str.replace(/, $/, '');
			str.length > maxLen && (str = str.substr(0, maxLen) + '...');

			funcName += ' \t// ' + str;
		}

		this.printWithIndent(func, funcName);
	},

	printWithIndent: function(func, funcName){
		typeof func.treeIndent !== 'undefined' && indent.setIndent(func.treeIndent);
		util.log(indent.getStr(), funcName);
	},

	log: function(){
		var args = Array.prototype.slice.call(arguments);
		var indentStr = indent.getStr(true);

		args.unshift(indentStr);
		util.log.apply(null, args);
	},

	reset: function(){
		indent.reset();
		nested.reset();
	}
};

module.exports = tree;
